package com.xy.redis;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.collections.map.JMap;
import com.xy.entity.*;
import com.xy.entity.join.SysMenuJoinSysRoleMenu;
import com.xy.mapper.SysMenuMapper;
import com.xy.mapper.SysRoleMapper;
import com.xy.mapper.SysSystemMapper;
import com.xy.utils.Beans;
import com.xy.utils.Emptys;
import com.xy.utils.RedisService;
import com.xy.utils.SpringBeanUtils;
import icu.mhb.mybatisplus.plugln.core.JoinLambdaWrapper;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class RedisCache {

    private static String getKey(Long loginId) {
        return String.format("%s:%d", "menu", loginId);
    }

    /**
     * 缓存权限相关到redis
     *
     * @param loginId 登录id
     */
    public static void cache(Long loginId) {
        SysSystemMapper sysSystemMapper = SpringBeanUtils.getBean(SysSystemMapper.class);
        SysRoleMapper sysRoleMapper = SpringBeanUtils.getBean(SysRoleMapper.class);
        SysMenuMapper sysMenuMapper = SpringBeanUtils.getBean(SysMenuMapper.class);
        RedisService<List<CacheEntity>> roleMenuRedisService = SpringBeanUtils.getBean(RedisService.class);
        //获取登录人所有系统
        List<SysSystem> sysSystems = sysSystemMapper.selectList(new LambdaQueryWrapper<SysSystem>()
                .eq(SysSystem::getSysUserId, loginId)
                .eq(SysSystem::getStatus, true)
        );
        if (!Emptys.check(sysSystems)) {
            return;
        }
        //获取登录人所有角色
        JoinLambdaWrapper<SysRole> sysRoleJoinLambdaWrapper = new JoinLambdaWrapper<>(SysRole.class)
                .eq(SysRole::getStatus, true)
                .innerJoin(SysSystemUserRole.class, SysSystemUserRole::getRoleId, SysRole::getId)
                .eq(SysSystemUserRole::getSystemUserId, loginId)
                .end();
        List<SysRole> sysRoles = sysRoleMapper.joinSelectList(sysRoleJoinLambdaWrapper, SysRole.class);
        JList<Long> sysRoleIds = new JArrayList<>(sysRoles).getProperty(SysRole::getId);
        if (sysRoleIds.size() == 0) {
            return;
        }
        //获取登录人所有角色菜单
        JoinLambdaWrapper<SysMenu> sysMenuJoinLambdaWrapper = new JoinLambdaWrapper<>(SysMenu.class)
                .eq(SysMenu::getStatus, true)
                .innerJoin(SysRoleMenu.class, SysRoleMenu::getMenuId, SysMenu::getId)
                .selectAs(cb -> cb
                        .add(SysRoleMenu::getSelectIsAdmin)
                        .add(SysRoleMenu::getRoleId)
                )
                .in(SysRoleMenu::getRoleId, sysRoleIds)
                .end();
        List<SysMenuJoinSysRoleMenu> sysMenus = sysMenuMapper.joinSelectList(sysMenuJoinLambdaWrapper, SysMenuJoinSysRoleMenu.class);
        JMap<Long, List<SysMenuJoinSysRoleMenu>> sysMenuMaps = new JArrayList<>(sysMenus).toMap(SysMenuJoinSysRoleMenu::getRoleId).group();
        sysSystems.forEach(sysSystem -> {
            List<CacheEntity> cacheEntitys = new ArrayList<>();
            sysRoles.forEach(sysRole -> {
                CacheEntity cacheEntity = new CacheEntity()
                        .setRoleId(sysRole.getId())
                        .setList(sysMenuMaps.get(sysRole.getId()));
                cacheEntitys.add(cacheEntity);
            });
            roleMenuRedisService.setMap(getKey(loginId), sysSystem.getId().toString(), cacheEntitys);
        });
    }

    /**
     * 获取菜单树
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenuTree(Long loginId, Long systemId) {
        //获取redis缓存
        RedisService<List<CacheEntity>> roleMenuRedisService = SpringBeanUtils.getBean(RedisService.class);
        List<CacheEntity> cacheEntities = roleMenuRedisService.getMap(getKey(loginId), systemId.toString());
        //角色菜单累加
        JList<SysMenuJoinSysRoleMenu> sysMenuJoinSysRoleMenus = new JArrayList<>();
        cacheEntities.forEach(cacheEntity -> sysMenuJoinSysRoleMenus.addAll(cacheEntity.getList()));
        //去重
        sysMenuJoinSysRoleMenus.comparing(SysMenuJoinSysRoleMenu::getId);
        //生成树
        JList<SysMenuJoinSysRoleMenu> tops = sysMenuJoinSysRoleMenus.filter().isNull(SysMenuJoinSysRoleMenu::getPaterId).list();
        for (SysMenuJoinSysRoleMenu top : tops) {
            List<SysMenuJoinSysRoleMenu> tree = Beans.tree(top, sysMenuJoinSysRoleMenus, SysMenuJoinSysRoleMenu::getId, SysMenuJoinSysRoleMenu::getPaterId, SysMenuJoinSysRoleMenu::getSonSysMenu);
            top.setSonSysMenu(tree);
        }
        return tops;
    }

    @Data
    @Accessors(chain = true)
    public static class CacheEntity {

        /**
         * 角色id
         */
        private Long roleId;

        /**
         * 菜单集合
         */
        private List<SysMenuJoinSysRoleMenu> list;

    }

}
