package com.xy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.SysSystemDto;
import com.xy.entity.SysSystem;
import com.xy.mapper.SysSystemMapper;
import com.xy.service.SysSystemService;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import com.xy.utils.SaTokenUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 系统表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2022-12-09
 */
@Service
@Api(tags = "系统接口")
@AllArgsConstructor
public class SysSystemServiceImpl extends ServiceImpl<SysSystemMapper, SysSystem> implements SysSystemService {

    @PostMapping("save")
    @ApiOperation("添加系统")
    public R<Long> save(@RequestBody @Validated SysSystemDto.Save save) {
        long count = count(new LambdaQueryWrapper<SysSystem>().eq(SysSystem::getCode, save.getCode()));
        if (count > 0) {
            return R.fail("标识已存在");
        }
        long loginId = Long.parseLong(SaTokenUtils.getId().toString());
        SysSystem sysSystem = copy(SysSystem.class, save)
                .create(loginId)
                .setSysUserId(loginId);
        save(sysSystem);
        return R.ok(sysSystem.getId());
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<SysSystemDto.Vo>> page(@RequestBody SysSystemDto.Select select) {
        long loginId = Long.parseLong(SaTokenUtils.getId().toString());
        LambdaQueryWrapper<SysSystem> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(select, SysSystem.class)
                .ge(SysSystem::getCreateTime, select.getBeginCreateTime())
                .le(SysSystem::getCreateTime, select.getEndCreateTime())
                .like(SysSystem::getName)
                .build();
        IPage<SysSystem> page = page(toIPage(select.getPage()), lambdaQueryWrapper);
        return R.ok(toPageBean(SysSystemDto.Vo.class, page));
    }
}
