package com.xy.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.crypto.SecureUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.SysSystemUserDto;
import com.xy.entity.SysSystemUser;
import com.xy.mapper.SysSystemUserMapper;
import com.xy.redis.RedisCache;
import com.xy.service.SysSystemUserService;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;

import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 权限用户表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2022-12-09
 */
@Service
@AllArgsConstructor
@Api(tags = "权限用户接口")
public class SysSystemUserServiceImpl extends ServiceImpl<SysSystemUserMapper, SysSystemUser> implements SysSystemUserService {

    @Override
    @ApiOperation("登录")
    public R<String> login(SysSystemUserDto.Login login) {
        //校验账密
        SysSystemUser sysSystemUser = getOne(new LambdaQueryWrapper<SysSystemUser>()
                .eq(SysSystemUser::getAccount, login.getAccount())
                .eq(SysSystemUser::getPassword, SecureUtil.md5(login.getPassword()))
        );
        if (sysSystemUser == null) {
            return R.fail("账号或密码错误");
        }
        if (!sysSystemUser.getStatus()) {
            return R.fail("账号已被封禁");
        }
        //生成token
        StpUtil.login(sysSystemUser.getId());
        //更新登录信息
        SysSystemUserDto.Update update = new SysSystemUserDto.Update()
                .setIsLogin(true);
        update.setId(sysSystemUser.getId());
        update(update);
        //缓存权限相关到redis
        RedisCache.cache(sysSystemUser.getId());
        return R.ok(StpUtil.getTokenValue());
    }

    @Override
    @ApiOperation("注册")
    public R<Boolean> register(SysSystemUserDto.Login login) {
        //校验重复
        long count = count(new LambdaQueryWrapper<SysSystemUser>()
                .eq(SysSystemUser::getAccount, login.getAccount())
        );
        if (count > 0) {
            return R.fail("账号已存在");
        }
        //注册
        SysSystemUser sysSystemUser = new SysSystemUser()
                .setAccount(login.getAccount())
                .setPassword(SecureUtil.md5(login.getPassword()))
                .create(null);
        save(sysSystemUser);
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<SysSystemUserDto.Vo>> page(@RequestBody SysSystemUserDto.Select select) {
        LambdaQueryWrapper<SysSystemUser> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(select, SysSystemUser.class)
                .ge(SysSystemUser::getCreateTime, select.getBeginCreateTime())
                .le(SysSystemUser::getCreateTime, select.getEndCreateTime())
                .build();
        IPage<SysSystemUser> page = page(toIPage(select.getPage()), lambdaQueryWrapper);
        return R.ok(toPageBean(SysSystemUserDto.Vo.class, page));
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R<Boolean> update(@RequestBody SysSystemUserDto.Update update) {
        SysSystemUser sysSystemUser = new SysSystemUser()
                .setId(update.getId())
                .setStatus(Emptys.check(update.getStatus()) ? update.getStatus() : null)
                .setPassword(Emptys.check(update.getIsAgainPwd()) && update.getIsAgainPwd() ? "88888888" : null)
                .setPassword(Emptys.check(update.getPassword()) ? SecureUtil.md5(update.getPassword()) : null);
        //如果是登录则更新登录时间和次数
        if (Emptys.check(update.getIsLogin()) && update.getIsLogin()) {
            sysSystemUser.setLastLoginTime(LocalDateTime.now()).setLoginNum(getById(update.getId()).getLoginNum() + 1);
        } else {
            sysSystemUser.update(Long.parseLong(SaTokenUtils.getId().toString()));
        }
        updateById(sysSystemUser);
        //如果封禁账号则踢下线
        if (Emptys.check(update.getStatus()) && !update.getStatus()) {
            StpUtil.logout(SaTokenUtils.getId());
        }
        return R.ok();
    }

}
